#include "DATA_Patterns.h"

//--------------------------------------------------------------------------
// class ID for Identification of a pattern
//--------------------------------------------------------------------------
ID::ID()
{
}

ID::ID(const ID& myid)
         :molRef(myid.molRef),strRef(myid.strRef),id(myid.id)
{
}

ID::ID(const string& myMolRef,const string& myStrRef,const string& myId)
      :molRef(myMolRef),strRef(myStrRef),id(myId)
{
}
 
ID::~ID()
{
}

const string& ID::getMolRef() const
{
   return molRef;
}

const string& ID::getStrRef() const
{
   return strRef;
}

const string& ID::getId() const
{
   return id;
}


//--------------------------------------------------------------------------
// a pattern consist of elements 
//--------------------------------------------------------------------------
SinglePattern::SinglePattern()
                              : Id(), pattern()
{
}

SinglePattern::SinglePattern(const SinglePattern &mySingP)
                              : Id(mySingP.Id),pattern(mySingP.pattern)
{
}


SinglePattern::~SinglePattern()
{
}
   
SinglePattern::SinglePattern(const string& myMolRef,const string& myStrRef,const string& myId,const intVec& mySinglePattern)
   : Id(myMolRef,myStrRef,myId),pattern(mySinglePattern) 
   { 
   }

const ID& SinglePattern::getID() const
   {
      return Id;
   }
   
const intVec& SinglePattern::getPat() const
   {
      return pattern;
   }
   
void SinglePattern::print()
{
   cout << "SinglePatternObject Info:" << endl;
   cout << "  Pattern " << biu::Util_String::intvec2str(getPat(),":")<< endl;
   cout << " ID MolRef: " << Id.getMolRef() << endl;
   cout << " ID StrRef: " << Id.getStrRef() << endl;
   cout << " ID id: " << Id.getId() << endl;
   cout << "---------------------------------" << endl;
}
   
//--------------------------------------------------------------------------
// class Pattern
//    this is the hole pattern with parts in different structure
//    therefore a Pattern consits of SinglePatterns 
//--------------------------------------------------------------------------
/*
Pattern::Pattern()
{
}

Pattern::Pattern(const Pattern& myP)
                  : id(myP.id),pattern(myP.pattern)
{
}

Pattern::~Pattern()
{
}

Pattern::Pattern(const SinglePattern& mySinglePat)
                  : id(mySinglePat.getID().getId())
{
   add(mySinglePat);
}  

const string& Pattern::getid() const
{
   return id;
} 

const SinglePattern& Pattern::getSinglePattern(const ID& myID) const
{
 //  if (pattern.find(myID)!=pattern.end())
      return pattern.find(myID)->second;
   // else 
     // {
       //  cerr << "SingPattern not found!" << endl;
      //   return pattern.;
     // }
}

const Pattern::selfMapTYPE& Pattern::getMap() const
{
   return pattern;
}

void Pattern::add(const SinglePattern& myPat)
{
   if (pattern.find(myPat.getID())==pattern.end())
      pattern.insert(make_pair(myPat.getID(),myPat));
}*/
//--------------------------------------------------------------------------
// class PatternMap
//    is able to manage an arbitrary bunch of patterns 
//--------------------------------------------------------------------------
/*      PatternMap::PatternMap()
      {
         patternMap.clear();
      }
   
      PatternMap::~PatternMap()
      {
      }
      const Pattern& PatternMap::getPattern(const string& id) const
      {
       //  if ( patternMap.find(id) != patternMap.end() )
            return patternMap.find(id)->second;
       //  else return Pattern();
      }
      
      void PatternMap::add(const SinglePattern& myPat)
      {        
        if (patternMap.find(myPat.getID().getId())!=patternMap.end())
            patternMap.find(myPat.getID().getId())->second.add(myPat);
        else patternMap.insert(make_pair(myPat.getID().getId(),Pattern(myPat)));  
      }
      void PatternMap::add(const Pattern& myPat)
      {
         patternMap.insert(make_pair(myPat.getid(),myPat));
      }
      
      const int PatternMap::size()
      {
         return patternMap.size();
      }
      
      const PatternMap::selfMapTYPE& PatternMap::getMap() const
      {  
         return patternMap;
      }
   */    
//--------------------------------------------------------------------------
// class PatternPairMap
//    is able to manage a pattern with 2 singlepatterns from pairwise_mcs 
//--------------------------------------------------------------------------
PatternPairMap::PatternPairMap()
{
   idMap.clear();
   patternList.clear();
   patternOrderedMap.clear();
}
    
PatternPairMap::~PatternPairMap()
{
}

void PatternPairMap::add(const string& id, 
                         const int& mysize, 
                         const SinglePattern& first, 
                         const SinglePattern& second)
{
   SelfValuePTR myP = SelfValuePTR(new PatternPair(id,mysize,first,second));
   patternList.push_back(myP);
   idMap.insert(make_pair(id,myP));
}

void PatternPairMap::add(const SelfValuePTR value)
{
   SelfValuePTR myP = SelfValuePTR(new PatternPair(*value));
   patternList.push_back(myP);
   idMap.insert(make_pair(value->getId(),myP));
}

void  PatternPairMap::makeOrderedMap()
{
   patternOrderedMap.clear();
   for(patListITER i = patternList.begin();i!=patternList.end();i++)
   {
      patternOrderedMap.insert(make_pair((*i)->getSize(),*i));
   }
}

void PatternPairMap::updateFromMap()
{
   if (!patternOrderedMap.empty())
   {
      idMap.clear();
      patternList.clear();
      for (orderedMapITER i=patternOrderedMap.begin();i!=patternOrderedMap.end();i++)
      {
         add(i->second);
      }
   }
}
const PatternPair& PatternPairMap::getPatternPair(const string& id)const
{
   return *(idMap.find(id)->second);
}

const    PatternPairMap::SelfValuePTR  PatternPairMap::getPatternPairPTR(const string& id)const
{
   return (idMap.find(id)->second);
}

const PatternPairMap::patListTYPE& PatternPairMap::getList()const
{
   return patternList;
}
const PatternPairMap::orderedMapTYPE& PatternPairMap::getOrderedMap() const
{
   return patternOrderedMap;
} 

PatternPairMap::orderedMapTYPE& PatternPairMap::getOrderedMap2()
{
   return patternOrderedMap;
}

const int PatternPairMap::size() const
{
   return idMap.size();
}
//--------------------------------------------------------------------------
// class SinglePatternMap
//    is able to manage an arbitrary bunch of single patterns 
//--------------------------------------------------------------------------
/*
SinglePatternMap::SinglePatternMap()
{
   singlePatternMap.clear();
}
      
SinglePatternMap::~SinglePatternMap()
{
}
      
const SinglePattern SinglePatternMap::getSinglePattern(const string& id) const
{
      return singlePatternMap.find(id)->second;
}
      
void  SinglePatternMap::add(const SinglePattern& mySingPat)
{
  // if (singlePatternMap.find(mySingPat.getID()) == singlePatternMap.end())
   singlePatternMap.insert(make_pair(mySingPat.getID().getId(),mySingPat));
 //  else cerr << "SinglePattern is already in SinglePatterMap! nozthing inserted!" << endl;
}

const SinglePatternMap::selfMapTYPE& SinglePatternMap::getMap()
{
   return singlePatternMap;
} 
*/
//--------------------------------------------------------------------------
// class SimplePatternList
//    is a simple list of SinglePatterns by their Id-Name 
//--------------------------------------------------------------------------
/*
SimplePatternMap::SimplePatternMap()
{
   simplePatternMap.clear();
}

SimplePatternMap::~SimplePatternMap()
{
}

void SimplePatternMap::add(const SinglePattern& mySingPat)
{
   simplePatternMap.insert(make_pair(mySingPat.getPat().size(),mySingPat));
}

const SimplePatternMap::selfMapTYPE&   SimplePatternMap::getMap()
{
   return simplePatternMap;
}
*/
//--------------------------------------------------------------------------
// class Molecule
//       stores all information for one molecule/sequence 
//--------------------------------------------------------------------------
Molecule::Molecule()
{
}

Molecule::Molecule(const string& myMolId, 
                   const string& mySeq, 
                   const string& myStrId, 
                   const string& myStr)
                   :id(myMolId),sequence(mySeq)
{
   structures.insert(make_pair(myStrId,myStr));
   actual_structure = myStrId;
   makeBasePairs(getStructure());
}

Molecule::Molecule(const Molecule& myMol)
                     :id(myMol.id),
                      sequence(myMol.sequence),
                      structures(myMol.structures),
                      basePairs(myMol.basePairs),
                      actual_structure(myMol.actual_structure)
{
}

Molecule::~Molecule()
{
}

const string& Molecule::getId() const
   {
      return id;
   }
   
const string& Molecule::getSequence() const
   {
      return sequence;
   }
   

const string& Molecule::getStructure() const
   {
  //     if (actual_structure != "nostructure")
         return (structures.find(actual_structure))->second;
   //   else 
    //     return NULL;
   }

const string& Molecule::getActualStructureId() const
   {
 //      if (actual_structure != "nostructure")
         return actual_structure;
  //    else 
     //    return NULL;
   }
   
void  Molecule::setActualStructureId(const string& strId)
{
   if (structures.find(strId) != structures.end())
      actual_structure = strId;
}
const char Molecule::getStructure(const unsigned int& position) const
   {
      if ((position>0)&&(position<=sequence.length()))
         return (structures.find(actual_structure)->second[position-1]);
      else return '#';
   }
   
const char Molecule::getBase(const unsigned int& position) const
   {
      if ((position>0)&&(position<=sequence.length()))
         return (sequence[position-1]);
      else return '#';
   }
   
const vector<int>& Molecule::getBasePairs() const
   {
         return ((basePairs.find(actual_structure))->second);
   }
    
const int Molecule::getBasePartner(const unsigned int& position) const
   {
      if ((position>0)&&(position<=sequence.length())&&(basePairs.find(actual_structure)->second[position-1]!=-1))
      
         return ((basePairs.find(actual_structure))->second[position-1])+1;
      else return -1;
   }

const int   Molecule::getLength() const

{
   return sequence.length();
}
    
/*void Molecule::add(const SinglePattern& myPat)
   {
      if ((myPat.getID().getMolRef()==this->getId())&&
          (structures.find(myPat.getID().getStrRef())!=structures.end()))    
               moleculePatterns.add(myPat);
      else 
         cerr << "Molecule:add:SingPat not inserted!" << endl;
}
   
const SinglePattern Molecule::getPattern(string myID) const
   {
     // return moleculePatterns.getSinglePattern(myID);
   }*/
   
/*SimplePatternMap& Molecule::getPatterns()
{
   typedef __gnu_cxx::hash_map<ID,SinglePattern,IDHash,IDEq>::const_iterator myIter;
      
   for (SinglePatternMap::selfMapCITER i=this->moleculePatterns.getMap().begin();i!=this->moleculePatterns.getMap().end();i++)
   {
      SinglePattern myS = i->second;
      if(myS.getID().getStrRef()==getActualStructureId())
         simplePatternMap.add(myS);
   }
   return simplePatternMap;
}*/
   
void Molecule::printInfo(const bool& shortview)
{
   cout << "Molecule Information:" << endl;
   cout << "-----------------------" << endl;
   cout << "Molecule/Sequence Id: " << getId() << endl;
   cout << "Structure Id: " << getActualStructureId() << endl;
   cout << "actual Structure: " << actual_structure << endl;
   cout << "Sequence length: " << getSequence().length() << endl;
   if (!shortview)
   {  
      cout << "Sequence: " << getSequence() << endl;   
      cout << "Structure: " << getStructure() << endl;
      cout << "BasePairs: " << biu::Util_String::intvec2str(getBasePairs(),",") << endl;
   }
   cout << "         ****" << endl<<endl;;
}
      
void Molecule::makeBasePairs(string myStr)
{
   vector<int> temp_bplist;
   stack<int> structure;
   temp_bplist.resize(myStr.size());
    for(unsigned int i=0;i<myStr.size();i++)
         if (myStr[i]=='(')
            structure.push(i);
        else 
            if (myStr[i]==')')
            {
               if (structure.empty())
               {
                  cerr << "...structure not correct. Please use a correct structure!" << endl;
                  exit(1);
               }
                  temp_bplist[i]=structure.top(); // index basepair partner
                  temp_bplist[structure.top()]=i; // index basepair partner
                  structure.pop();
            }
            else
            temp_bplist[i]=-1;      // not paired
   if (!structure.empty())
   {
      cerr << "...structure not correct. Please use a correct structure!" << endl;
      actual_structure = "nostructure";
      exit(1);
   }
   basePairs.insert(make_pair(actual_structure,temp_bplist));
}

