#ifndef MCSERPAP_H_
#define MCSERPAP_H_

#include "mcsEnsemble.h"
#include "ExpaRNAOptions.h"
#include <map>
#include <algorithm>
#include <iostream>

class mcsERPAP
{
    public:

    					mcsERPAP(	const	Molecule&       myMol1,
    								const 	Molecule&       myMol2,
    								const 	PatternPairMap& myPatterns,
    										PatternPairMap& myERPAP,
    										int&		    mySize,
    								const	ExpaRNAOptions& myOptions)

    	                                   :matchedPatterns(myERPAP),
    	                                	mol1(myMol1),mol2(myMol2),
    	                                    patterns(myPatterns),
    	                                    matchedSize(mySize),
    	                                    minPatternSize(myOptions.minPatternSize),
    	                                    lcsOptions(myOptions) {};

        virtual 		~mcsERPAP();

        		void    calculateERPAP();

	private:

		enum 		KeyTypeE	{BEGIN_KEY, END_KEY };
        struct      HoleKeyS;

        struct HoleKeyS
        {
        	PatternPairMap::SelfValuePTR    pattern;
            intPPair                        bounds;
        };

        typedef     HoleKeyS                            HoleKey;
        typedef     HoleKey*                            HoleKeyPTR;
        typedef     multimap<int,HoleKeyPTR>            HoleOrderingMapTYPE;
        typedef     HoleOrderingMapTYPE::const_iterator HoleMapCITER;

        void    preProcessing				();
        void    calculateHoles				();
		//    void    calculatePatternBoundaries	(const boost::shared_ptr<PatternPair>& myPair);
        void    calculatePatternBoundaries	(PatternPair* myPair);
        void 	calculateTraceback			(const int i,const int j,const int k,const int l,vector < vector<int> > holeVec);
        int 	D_rec						(const int& i,const  int& j,const int& k,const int& l,vector < vector<int> >& D_h);
        int 	max3						(int& a, int& b, int& c);

				vector< vector<PatternPairMap::SelfValuePTR> >	EPM_Table;
				PatternPairMap&									matchedPatterns;
				HoleOrderingMapTYPE    					 		holeOrdering;
        const 	Molecule&					               		mol1,mol2;
        const 	PatternPairMap&         						patterns;
				int& 											matchedSize;
		const	int                     						minPatternSize;
				ExpaRNAOptions									lcsOptions;
};
#endif /*MCSERPAP_H_*/
